/**
 * Get current timestamp in milliseconds
 *
 * @category Utilities
 * @returns {number}
 */
export function now(): number;
/**
 * Random number generator
 * @example
 * ```javascript
 * 	random('seed') // 0.9367527104914188
 * ```
 *
 * @category Utilities
 * @param {string} seed
 * @param {number} min
 * @param {number} max
 * @param {number} decimals
 * @returns {number}
 */
export function random(seed?: string | number | null, min?: number, max?: number, decimals?: number): number;
/**
 * Evenly distributes a number of points in a buffer
 *
 * @param {Float32Array} buffer current buffer
 * @param {number} pointsToAdd points to add
 * @return {*}  {Float32Array}
 */
export function distributePointsInBuffer(buffer: Float32Array, pointsToAdd: number): Float32Array;
/**
 * Leads two buffers to have the same number of points
 *
 * @param from
 * @param to
 * @returns
 */
export function prepareBuffersForInterpolation(from: Float32Array, to: Float32Array): [Float32Array, Float32Array];
/**
 * Interpolate two buffer
 *
 * @param from
 * @param to
 * @param offset
 * @returns
 */
export function interpolate(from: Float32Array, to: Float32Array, initialOffset?: number | Array<number>): Float32Array;
export const PI2: number;
/**
 * Convert number from radians to degrees
 *
 *
 * @example
 * ```javascript
 * toDegrees(Math.PI) // 180
 * ```
 *
 * @param {number} radians
 * @returns {number}
 */
export function toDegrees(radians: number): number;
/**
 * Convert angle from degrees to radians
 * @example
 * ```javascript
 * toRadians(180) // 3.141592653589793
 * ```
 *
 * @param {number} degrees
 * @returns {number}
 */
export function toRadians(degrees: number): number;
/**
 * Linear interpolation from `a` when `i` as 0 an `b` when `i' as 1
 *
 * @param {number} a
 * @param {number} b
 * @param {number} i
 * @returns {number}
 */
export function lerp(a: number, b: number, i: number): number;
/**
 * Return number between 0 and 1
 *
 * @export
 * @param {number} value
 * @return {*}  {number}
 */
export function clamp01(value: number): number;
/**
 * Return number between min and max
 *
 * @example
 * ```javascript
 * clamp(0, 1, 1.2) // 1
 * clamp(0, 1, -2) // 0
 * ```
 * @param {number} min
 * @param {number} max
 * @param {number} value
 * @returns {number}
 */
export function clamp(min: number, max: number, value: number): number;
/**
 *
 *
 * @export
 * @param {number} val
 * @param {number} [decimals=0]
 * @return {*}  {number}
 */
export function round(val: number, decimals?: number): number;
/**
 * Map number between refMin e refMax from min and max
 *
 *
 * @example
 * ```javascript
 * relativeClamp(0, 1, 0.5, 100, 200) // 150
 * ```
 *
 * @param {number} refMin
 * @param {number} refMax
 * @param {number} value
 * @param {number} toMin
 * @param {number} toMax
 * @returns {number}
 */
export function relativeClamp(refMin: number, refMax: number, value: number, toMin: number, toMax: number): number;
/**
 *
 *
 * @export
 * @param {number} a
 * @param {number} b
 * @param {number} [epsilon=128 * Number.EPSILON]
 * @param {*} [abs_th=Number.MIN_SAFE_INTEGER]
 * @return {*}
 */
export function nearly(a: number, b: number, epsilon?: number, abs_th?: number): boolean;
/**
 * <a href="https://github.com/jwagner/simplex-noise.js" target="_blank">SimplexNoise</a>
 * Use 'random' as seed property for random seed.
 * Return value between -1 and 1
 *
 * @category Utilities
 *
 * @param {string} [seed='random']
 * @param {number} [x=0]
 * @param {number} [y=0]
 * @param {number} [z=0]
 * @param {number} [min=-1]
 * @param {number} [max=-1]
 * @returns {number} between -1 and 1
 */
export function noise(seed?: string, x?: number, y?: number, z?: number, min?: number, max?: number): number;
export function createCanvas(width: number, height: number): HTMLCanvasElement;
export type pixel = [number, number, number, number];
export type ImageDataSource = HTMLCanvasElement | ImageData;
export function eachPixel(canvasOrImageData: ImageDataSource, callback: (pixel: pixel, x: number, y: number, count: number) => void | pixel): ImageData;
export function pixelAt(canvasOrImageData: ImageDataSource, x: number, y: number, at?: number[]): pixel;
export function pixelAtTop(canvasOrImageData: ImageDataSource, x: number, y: number, distance?: number): pixel;
export function pixelAtBottom(canvasOrImageData: ImageDataSource, x: number, y: number, distance?: number): pixel;
export function pixelAtLeft(canvasOrImageData: ImageDataSource, x: number, y: number, distance?: number): pixel;
export function pixelAtRight(canvasOrImageData: ImageDataSource, x: number, y: number, distance?: number): pixel;
export function pixelAtTopLeft(canvasOrImageData: ImageDataSource, x: number, y: number, distance?: number): pixel;
export function pixelAtTopRight(canvasOrImageData: ImageDataSource, x: number, y: number, distance?: number): pixel;
export function pixelAtBottomLeft(canvasOrImageData: ImageDataSource, x: number, y: number, distance?: number): pixel;
export function pixelAtBottomRight(canvasOrImageData: ImageDataSource, x: number, y: number, distance?: number): pixel;
export type fit = 'cover' | 'contain' | 'none';
export function fit(sourceWidth: number, sourceHeight: number, destWidth: number, destHeight: number, fit: fit): {
    x: number;
    y: number;
    width: number;
    height: number;
    scale: number;
};
export function loadImage(src: string): Promise<HTMLImageElement>;
export function fitImage(image: HTMLImageElement, width: number, height: number, mode?: fit): {
    x: number;
    y: number;
    width: number;
    height: number;
    scale: number;
};
/**
 * number between 0 and 255
 */
export type rgb = [number, number, number];
export type rgba = [number, number, number, number];
/**
 * number between 0 and 360, 0 and 100, 0 and 100
 */
export type hsl = [number, number, number];
/**
 * Convert RGB to HSL
 *
 * @export
 * @param {rgb} [r, g, b]
 * @return {*}  {hsl}
 */
export function rgbToHsl([r, g, b]: rgb): hsl;
/**
 *  * Convert HSL to RGB
 *
 * @export
 * @param {hsl} [h, s, l]
 * @return {*}  {rgb}
 */
export function hslToRgb([h, s, l]: hsl): rgb;
export function rgbToHex([r, g, b]: rgb): string;
/**
 * Brightness [0-255]
 *
 * @export
 * @param {rgb} [r, g, b]
 * @return {*}  {number from 0 to 255}
 */
export function brightness([r, g, b, a]: rgb | rgba): number;
/**
 * Luminance [0-1]
 *
 * @export
 * @param {rgb} [r, g, b]
 * @return {*}  {number from 0 to 1}
 */
export function luminance([r, g, b]: rgb): number;
export function isDark([r, g, b]: rgb): boolean;
export function isLight([r, g, b]: rgb): boolean;
/**
 * vec2 operation
 *
 */
export const vec2: {
    /**
     * from new vertex
     *
     * @param {Array<number> | number} [x=0]
     * @param {number} [y]
     * @returns {Array<number>}
     */
    from: (x?: Array<number> | number, y?: number) => Array<number>;
    fromAngle: (angle: number, length?: number) => Array<number>;
    normalize: (v: Array<number>) => Array<number>;
    /**
     * Distance between two points
     *
     * @param {Array<number>} a
     * @param {Array<number>} b
     * @returns {number}
     */
    distance: (a: Array<number>, b: Array<number>) => number;
    /**
     * dot product
     *
     * @param {Array<number>} a
     * @param {Array<number>} b
     * @returns {number}
     */
    dot: (a: Array<number>, b: Array<number>) => number;
    /**
     * length of point
     *
     * @param {Array<number>} vec
     * @returns {number}
     */
    length: (vec: Array<number>) => number;
    /**
     * angle between two point
     *
     * @param {Array<number>} a
     * @param {Array<number>} b
     * @returns {number}
     */
    angle: (a: Array<number>, b: Array<number>) => number;
    /**
     * skewX point
     *
     * @param {Array<number>} vec
     * @param {number} m
     */
    skewX: (vec: Array<number>, m: number) => void;
    /**
     * skewY point
     *
     * @param {Array<number>} vec
     * @param {number} m
     */
    skewY: (vec: Array<number>, m: number) => void;
    /**
     * squeezeX point
     *
     * @param {Array<number>} vec
     * @param {number} m
     */
    squeezeX: (vec: Array<number>, m: number) => void;
    /**
     * squeezeY point
     *
     * @param {Array<number>} vec
     * @param {number} m
     */
    squeezeY: (vec: Array<number>, m: number) => void;
    /**
     * Rotate point
     *
     * @param {Array<number>} vec
     * @param {Array<number>} MATRIX
     * @param {Array<number>} fromPoint
     * @internal
     */
    rotate: (vec: Array<number>, MATRIX: Array<number>, fromPoint: Array<number>) => void;
    /**
     * RotateX point
     *
     * @param {Array<number>} vec
     * @param {Array<number>} fromPoint
     * @param {number} rad
     */
    rotateX: (vec: Array<number>, fromPoint: Array<number>, rad: number) => void;
    /**
     * RotateY point
     *
     * @param {Array<number>} vec
     * @param {Array<number>} fromPoint
     * @param {number} rad
     */
    rotateY: (vec: Array<number>, fromPoint: Array<number>, rad: number) => void;
    /**
     * RotateZ point
     *
     * @param {Array<number>} vec
     * @param {Array<number>} fromPoint
     * @param {number} rad
     */
    rotateZ: (vec: Array<number>, fromPoint: Array<number>, rad: number) => void;
    /**
     * Translate vertex
     *
     * @param {Array<number>} vec
     * @param {Array<number>} to
     */
    translate: (vec: Array<number>, to: Array<number>) => void;
    /**
     * Scale vertex
     *
     * @param {Array<number>} vec
     * @param {Array<number>} to
     */
    scale: (vec: Array<number>, to: Array<number>) => void;
    /**
     * Scale vertex
     *
     * @param {Array<number>} vec
     * @param {Array<number>} to
     */
    divide: (vec: Array<number>, to: Array<number>) => void;
    /**
     * Vec to string
     *
     * @param {Array<number>} vec
     * @return {string}
     */
    toString: (vec: Array<number>) => string;
    /**
     * Vertex [0, 0]
     */
    ZERO: [number, number];
    /**
     * Vertex [1, 1]
     */
    ONE: [number, number];
};
/**
 * Random element from array
 *
 * @param {Array<any>} a
 * @param {number} index
 * @returns {Aany}
 */
export function randomElement(a: Array<any>): any;
/**
 * Create array with value
 *
 * @export
 * @param {number} size
 * @param {((i: number, array: Array<any>) => any | any)} value
 * @return {*}
 */
export function array(size: number, value: (i: number, array: Array<any>) => any | any): any[];
/**
 * Generate matrix
 *
 * @export
 * @param {number} size
 * @param {((i: number, j: number, matrix: Array<Array<any>>) => any | any)} value
 * @return {*}
 */
export function matrix(rows: number, cols: number, value: (i: number, j: number, matrix: Array<Array<any>>) => any | any): any[][];
/**
 * Each matrix and return a new matrix
 *
 * @export
 * @param {Array<Array<any>>} matrix
 * @param {(rowIndex: number, colIndex: number, matrix: Array<Array<any>>) => any | any} value
 * @return {*}  {Array<Array<any>>}
 */
export function eachMatrix(matrix: Array<Array<any>>, value: (value: any, rowIndex: number, colIndex: number, matrix: Array<Array<any>>) => any | any): Array<Array<any>>;
/**
 * Return angle (atan) from offset (or center) for matrix repetition.
 * Offset is array between [-1, -1] and [1, 1].
 * The return value is between -Math.PI / 2 and Math.PI / 2
 *
 * @param {Array<Array<any>>} matrix
 * @param {[number, number]} offsetFromCenter
 * @returns {number} between -Math.PI / 2 and Math.PI / 2
 */
export function angleFromMatrix(matrix: Array<Array<any>>, rowIndex: number, colIndex: number, offsetFromCenter?: [number, number]): number;
/**
 * Return angle (atan2, 4 quadrants) from offset (or center) for matrix repetition.
 * Offset is array between [-1, -1] and [1, 1].
 * The return value is between -Math.PI an Math.PI
 *
 * @param {Array<Array<any>>} matrix
 * @param {[number, number]} offsetFromCenter
 * @returns {number} between -Math.PI an Math.PI
 */
export function angle2FromMatrix(matrix: Array<Array<any>>, rowIndex: number, colIndex: number, offsetFromCenter?: [number, number]): number;
/**
 * Return distance from offset (or center) for matrix repetition.
 * The return value is between 0 and 1
 *
 * @param {Array<Array<any>>} matrix
 * @param {[number, number]} offsetFromCenter offset relative to distance prop
 * @returns {number} between 0 and 1
 */
export function distanceFromMatrix(matrix: Array<Array<any>>, rowIndex: number, colIndex: number, offsetFromCenter?: [number, number]): number;
export type line = [number, number, number, number];
export function lineIntersects(lineA: line, lineB: line): boolean;
export function polylineIntersects(polylineA: number[], polylineB: number[]): boolean;
export type point = [number, number];
/**
 * Check point is inside a polygon
 *
 * @links https://www.algorithms-and-technologies.com/point_in_polygon/javascript
 * @export
 * @param {point} point
 * @param {number[]} polygon
 * @return {*}
 */
export function pointInPolygon(polygon: number[], [x, y]: point): boolean;
export function pointInRect(point: point, rect: [number, number, number, number]): boolean;
export interface ZIPEvents {
    /**
     * Called when start ZIP rendering
     */
    start?: (props: {
        chunks: number;
        totalFrames: number;
        framesForChunk: number;
    }) => void;
    /**
     * Called each frame render
     */
    progress?: (props: {
        chunk: number;
        frame: number;
        totalFrames: number;
        framesForChunk: number;
        totalChunks: number;
        renderTime: number;
        remainingTime: number;
        elapsedTime: number;
    }) => void;
    prepare: CallableFunction;
}
export interface VideoEvents {
    /**
     * Called when start video rendering before loading FFmpeg.wasm
     */
    init?: (props: {
        totalFrames: number;
        duration: number;
        framerate: number;
        type: TRendererVideoType;
    }) => void;
    /**
     * Called when FFmpeg.wasm is loaded
     */
    start?: (props: {
        totalFrames: number;
        duration: number;
        framerate: number;
        type: TRendererVideoType;
    }) => void;
    /**
     * Called each frame render
     */
    progress?: (props: {
        frame: number;
        duration: number;
        totalFrames: number;
        renderTime: number;
        remainingTime: number;
        elapsedTime: number;
    }) => void;
    /**
     * Called when each frame is rendered and video generation start
     */
    preparing?: CallableFunction;
}
export type TRendererVideoType = 'video/webm' | 'video/mp4' | 'gif';
/**
 * Blob or Buffer
 *
 * @category Renderer
 */
export type BoB = Blob | Buffer;
/**
 * Options or Quality
 *
 * @category Renderer
 */
/**
 * The Renderer is a class for exporting the scene
 *
 * @category Renderer
 * @class Renderer
 * @extends {Emitter<IRendererEvents>}
 */
export class Renderer {
    constructor(ffmpegCorePath?: string);
    /**
     * Remove all stored frames
     *
     */
    flush(): void;
    /**
     * Render any frame and create array of zip
     *
     * @param imagesType
     * @param quality
     * @param framesForChunk
     * @returns
     */
    zip(imagesType?: 'image/jpeg' | 'image/png', framesForChunk?: number, events?: ZIPEvents): Promise<Array<Uint8Array>>;
    /**
     * Render animation
     *
     * @param type render type
     * @param quality
     * @param ffmpegLogger
     * @param ffmpegProgress
     * @returns
     */
    video(type: TRendererVideoType | undefined, framerate: number | undefined, events: VideoEvents & {
        ffmpegLogger?: (logParams: {
            type: string;
            message: string;
        }) => any;
        ffmpegProgress?: (progressParams: {
            ratio: number;
        }) => any;
    }): Promise<Uint8Array>;
    /**
     * Render frame `frameNumber` to Blob or Buffer
     *
     * @param mime image type
     * @param options quality or options
     */
    store(canvas: HTMLCanvasElement, mime: 'image/png' | 'image/jpeg', quality?: number): void;
    /**
     * Canvas to DataURL
     *
     * @param mime
     * @param optionsOrQuality
     * @returns
     */
    toDataUrl(canvas: HTMLCanvasElement, mime: 'image/png' | 'image/jpeg', optionsOrQuality?: number): null | string;
}

//# sourceMappingURL=types.d.ts.map
